function L = getNLoperator(f, choice)

[Ny,Nx] = size(f); % image size
n = Ny*Nx; % nb of nodes
Im0 = f/ max(f(:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute NL-Weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-----------------------------------------------
% SY - \in (1,max_neighbors); size(SY) = size(image) ---> number of
% edges = sum(SY(:)) - probably denotes size of pixel's neighbourhood
% (probably does not take into account symmetry)
%-----------------------------------------------
% W - \in (-1,1); size(W) =  size(image) x 2 NbNeigh ; probably w_(i,j) = sqrt (W_(i,j)) 
% Y - \in(0,max(size(image))-1); size(Y) =  size(image) x 2 NbNeigh ;
%-----------------------------------------------

% parameters for Weight function
m = 5; % patch size 5
w = 11; % window search size 11
h = (0.33)^2; % scale parameter for weight function %0.33 state of the art 
NbNeigh = 10; % number of neighbors
YES  =0;
NO = 1;
includeCloseNeigh = YES; % include 4 closest neighbors
includeCloseNeigh = NO;
VecParametersW = [ Ny; Nx; m; w; h; NbNeigh; includeCloseNeigh; ];

tic
[W,Y,SY] = compute_fastNLWeights_mex(single(Im0),...
    single(VecParametersW));
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute weights and graph coloring
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

max_neighbors=NbNeigh*2+(1-includeCloseNeigh)*8;
iNbNeigh4 = 4*NbNeigh;
iNbNeigh2 = 2*NbNeigh;
nbedges=sum(SY(:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute: neighbors, weights, degree,  normalization_factor=zeros
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% degree(i) - \in (1,max_neighbors); size(degree) = size(im(:));
% max position of non-zero weight
% neighbors(i,k) - size(neighbors) = size(im(:)) x max_neighbors; 
%-----------------------------------------------
% normlization factor - size(normlization factor) = size(im(:)); 
% normlization factor_i = sum_i w_{i,j}
%-----------------------------------------------
% gt - (exacly the same as normalization factor)
%-----------------------------------------------
% weights - size(neighbors) = size(im(:)) x max_neighbors; 
%-----------------------------------------------

neighbors= (nbedges+1)*ones(n, max_neighbors);
weights= zeros(n, max_neighbors);
degree=zeros(n,1);
normalization_factor=zeros(n,1);

for iy=0:Ny-1
    for ix=0:Nx-1
        
        iX = (ix)*Ny+ (iy);
        iXb= (iy)*Nx+ (ix);
        iNbNeighX = SY(iXb+1);
        iXNeigh4 = iX*iNbNeigh4;
        iXNeigh2 = iX*iNbNeigh2;
        for i=0:iNbNeighX-1
            ixY = Y(iXNeigh4+2*i+1);
            iyY = Y(iXNeigh4+2*i+1+1);
            iY = ixY*Ny+iyY;
            
            neighbors(ceil(iXNeigh2/iNbNeigh2)+1, i+1)= iY+1;
            weights(ceil(iXNeigh2/iNbNeigh2)+1, i+1)= sqrt(W(iXNeigh4+2*i+1));
            normalization_factor(ceil(iXNeigh2/iNbNeigh2)+1) = normalization_factor(ceil(iXNeigh2/iNbNeigh2)+1)+W(iXNeigh4+2*i+1); % sum of all weights associated with given pixel
            %gt(ceil(iXNeigh2/iNbNeigh2)+1)= gt(ceil(iXNeigh2/iNbNeigh2)+1)+W(iXNeigh4+2*i+1);
        end
        degree(ceil(iXNeigh2/iNbNeigh2)+1)=iNbNeighX;
        
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute number of edges
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% nb_edges - number of edges (i < neighbours(i,k)  )
%-----------------------------------------------

nb_nonedges = 0;
nb_edges= 0;
for i=1:n % n - number of pixels in image
    for k=1:degree(i) % degree - max non-zero weight
        if i<neighbors(i,k),
            nb_edges=nb_edges+1;
        else
            nb_nonedges = nb_nonedges+1;
        end
    end
end
%neighbors(neighbors==nbedges+1)=nb_edges+1;

neighbors(neighbors==nbedges+1)=n+1;
nb_nonedges;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sort weights in incraing order
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% weights1 = weights';
% neighbors1 = neighbors';
% [B,IX] = sort(weights1);
% for j = 1:n
% 	C(:,j) =  neighbors1(IX(:,j),j);
% end
% weights = B';
% neighbors = C';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% find linear operator L = [L1, L2, ..., Lk]^\top
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if choice == 0
    
    k = max(degree(:));
    i = [1:n,1:n];
    L = ones(0,0);
    nneighbors = neighbors';
    for iter = 1:k
        j = [1:n, nneighbors(iter,:)]; % to be updated 
        s = [weights(:,iter), - weights(:,iter)];
        temp = sparse(i,j,s,n,n+1); %Build sparse matrix
        ind = find(temp(:)~=0);
        if (~isempty(ind))
            L = [L; temp(1:n,1:n)];
        end
    end	
else
   tic
   [color, edges, nbcolor, weightmatrix, neighbors] = split_arbitrary_graph(neighbors, degree, weights);
   L=incidence(edges, weightmatrix); 
   toc
end



