function A=incidence(edges,weights,N)
%Function A=incidence(edges,weights,N) computes the edge-node incidence 
%matrix of the graph specified by edges
%
%Inputs:    edges - A Mx2 list of M edges
%           weights - Optional Mx1 edge weights 
%               (defaults to all ones if not specified)
%           N - Optional number of nodes in the graph.  Used if
%               N > max(edges) (i.e., isolated nodes are present)
%
%Outputs:   A - The edge-node incidence matrix
%
%

%Initialization
M=size(edges,1);

%Check for weights
if nargin < 2
    weights=ones(M,1);
end

%Check for isolated nodes
if nargin < 3
    N=max(max(edges));
end

%Build sparse incidence matrix
%i = [1:M,1:M]
%j = [edges(:,1),edges(:,2)]
%s = [weights,-weights]
%A - spase matrix 
%size(A) = (M,N) 
%A(i(k),j(k)) = s(k)
%nzmax = length(s) - maximum numer of non-zero
A=sparse([1:M,1:M],[edges(:,1),edges(:,2)],[weights,-weights],M,N);
