function mu = golden_search(crit,mu_inf,mu_sup)

%Minimisation de la fonction crit(mu) par l'algorithme du nombre d'or


%ENTREES
%crit : fonction handle matlab renvoyant crit(mu) 
%mu_inf,mu_sup : valeurs minimales et maximales acceptables pour mu

%SORTIE
%mu : minimise crit(mu) sur l'intervalle [mu_inf,mu_sup]

%EXEMPLE
%mu = golden_search(@(mu) cos(mu),0,3*pi/2)

tau     = 0.38197;
tol     = 1e-3;
maxiter = -2.078*log(tol);

mu_1 = (1-tau)*mu_inf + tau*mu_sup;
mu_2 = tau*mu_inf + (1-tau)*mu_sup;

J_1 = feval(crit,mu_1);
J_2 = feval(crit,mu_2);

disp(['Initialisation : Test de mu_1 = ',num2str(mu_1),' et mu_2 = ',num2str(mu_2)]);

iter = 1;
while iter<maxiter
    if J_1>=J_2
        mu_inf = mu_1;  mu_1   = mu_2;  J_1 = J_2;
        mu_2 = tau * mu_inf + (1-tau)*mu_sup;
        J_2 = feval(crit,mu_2);
        disp(['it = ',num2str(iter),' | mu = ',num2str(mu_2),' | crit(mu) = ',num2str(J_2)]);
    else
        mu_sup = mu_2;
        mu_2   = mu_1;
        J_2 = J_1;
        mu_1  = (1-tau)*mu_inf + tau*mu_sup;
        J_1 = feval(crit,mu_1);
        disp(['it = ',num2str(iter),' | mu = ',num2str(mu_1),' | crit(mu) = ',num2str(J_1)]);
    end
   
    iter = iter+1;
end

if J_1>J_2,
    mu = mu_2; 
else
    mu = mu_1; 
end
 