%------------------------------------------------------- %
% Primal-Dual splitting algorithm [Combettes et al. 2011]%
%       Poisson-Gaussian data-fidelity term                       %
%------------------------------------------------------- %


function  x = primaldualrecPG_Hyb(x0,y,yTrunc,h,NbIt,nx,ny, gvar, gmean, alpha,lambda1, lambda2, xmin,xmax,I, data_fidelity, prior, L1, L2, degree,gamma,tolerance)
             % primaldualrecPG_Hyb(x0,y,h,NbIt,nx,ny,v,m,alpha,p(1)^2,p(2)^2, xmin,xmax,I,data_fidelity,prior, VHes, VTV, gamma);
                             

% Vhes - >L1
% Vtv ->L2
% ----------------------------------------------------------------------- %
% f(x) = fun(Hx,y) + i_[xmin,xmax} + lambda1 * funProxPrior1(L1x) + lambda2 * funProxPrior2(L2x) 
% ----------------------------------------------------------------------- %

affiche = 500; %frequency display
x = x0; %Initialization primal variable
v1 = zeros(size(L1,1),1);   %Initialization dual variable 1
v2 = zeros(size(L2,1),1);   %Initialization dual variable 2
L1t = L1'; % transposition of the linear operator L1
L2t = L2'; % transposition of the linear operator L2


switch data_fidelity
    case 'PoissGauss'
        tabn = inittabn(y,gvar,gmean,alpha, tolerance);
        taby = inittaby(y,gvar,gmean,alpha);
        funGrad = @gradPGp; %(Hx,y,taby,tabn,gvar,gmean,alpha);
        algo = 'GradProx';
    case 'Gauss'
        tabn = 0;
        taby = 0;
        funGrad = @grad_gaussiandep; %(x,y,h,gvar,alpha, nx,ny)
        algo = 'GradProx';
    case 'GAST'
        tabn = 0;
        taby = 0;
        funGrad = @grad_gaussiangast; %(x,y,h, gvar,nx,ny); % grad GAST  
        algo = 'GradProx';
  case 'Poiss'
        algo = 'ProxProx';
        
        funProx = @prox_poisson; %(1/gamma, y22./gamma,y,v);   
        
    case 'SPoiss'    
        algo = 'ProxProx';
        funProx = @prox_poissonnization; %(1/gamma, y22./gamma,y,v);
    case 'EXP'
        tabn = 0;
        taby = 0;
        funGrad = @gradBenvp; %(Tx,y,taby,tabn, gvar,gmean, alpha)   
        algo = 'GradProx';
    case 'WL2'
        tabn = 0;
        taby = 0;
        funGrad = @grad_weightedl2; %(Tx,y,taby,tabn, gvar,gmean, alpha)   
        algo = 'GradProx';
end

switch prior
    case 'HTV'
        funProxPrior1 = @prox_hes; %(1/gamma,y21/gamma,lambda1,nx,ny);
        funProxPrior2 = @prox_tv; %(1/gamma,y22/gamma,lambda2,nx,ny);
end


% ---------------------------------------------------------- %
% ----- ----- ----- ----- START ALGO ----- ----- ----- ----- %
% ---------------------------------------------------------- %
switch algo
    
    case 'ProxProx'
        
        v3 = zeros(nx*nx,1);  
        y = max(y,0.0001);
        for k = 1:NbIt
            Hv3 = ConvolutionAdj2D(v3,h,nx,ny);
            Gsv1 = L1t*v1;
            Gsv2 = L2t *v2;
            y1 = x - gamma * (Gsv1 + Hv3 + Gsv2);  % line 1
            p1 = max(min(y1,xmax),xmin);  % line 2
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gx = (x'*L1t)'; %m =1 (Hessian prox loop)    
            y21 = v1 + gamma *Gx;
            prox = funProxPrior1(1/gamma,y21/gamma,lambda1,nx,ny);
            p21 = y21 - gamma  * prox;
            Gp1 = (p1'*L1t)';
            q21 = p21 + gamma * Gp1;
            v1  = v1 - y21 + q21;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gx = L2 * x;   %m = 2 (TV prox loop)
            y22 = v2 + gamma *Gx;
            prox = funProxPrior2(y22/gamma,degree,lambda2/gamma,nx,ny); %(x,degree, coeff,nx,ny)
            p22 = y22 - gamma  * prox;
            Gp2 = L2*p1; 	
            q22 = p22 + gamma * Gp2;
            v2  = v2 - y22 + q22;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Hx  = Convolution2D(x,h,nx,ny); %m = 3 (Poisson prox loop)
            y23 = v3 + gamma * Hx;
            p23 = y23-gamma*funProx(1/gamma, y23./gamma,yTrunc,gvar);
            Hp1 = Convolution2D(p1,h,nx,ny);              
            q23 = p23 + gamma*Hp1;
            v3  = v3 - y23 + q23;    
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gsp21 = L1t*p21; % q1 update
            Gsp22 = L2t*p22;
            Hp23  = ConvolutionAdj2D(p23,h,nx,ny);  
            q1 = p1 - gamma * (Gsp21 + Hp23 + Gsp22);
            x = x - y1 + q1;% line 8
            %display
            if(mod(k,affiche)==1)
                SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
                MAE = mean(abs(I(:)-x) );
                disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]);  
            end
        end
        % compute snr
        SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
        MAE = mean(abs(I(:)-x) ); 
        
    case 'GradProx'
        
        for k = 1:NbIt
            
            Gsv1 = L1t*v1; %line 1
            Gsv2 = L2t*v2;
            Hx = Convolution2D(x,h, nx,ny); 
            GradHx1 = funGrad(Hx,yTrunc,taby,tabn,gvar,gmean,alpha,tolerance);
            GradHx = ConvolutionAdj2D(GradHx1,h,nx,ny);
            y1 = x - gamma *(Gsv1 + Gsv2+GradHx);
            p1 = max(min(y1,xmax),xmin);      
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gx = (x'*L1t)'; 
            y21 = v1 + gamma *Gx;
            prox = funProxPrior1(1/gamma,y21/gamma,lambda1,nx,ny);
            p21 = y21 - gamma  * prox;
            Gp1 = (p1'*L1t)';
            q21 = p21 + gamma * Gp1;
            v1  = v1 - y21 + q21;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gx = L2 * x;   
            y22 = v2 + gamma *Gx;
            prox = funProxPrior2(y22/gamma,degree,lambda2/gamma,nx,ny); %(x,degree, coeff,nx,ny)
            p22 = y22 - gamma  * prox;
            Gp1 = L2*p1; 	
            q22 = p22 + gamma * Gp1;
            v2  = v2 - y22 + q22;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Gsp21 = L1t*p21; 
            Gsp22 = L2t*p22; 
            Hp1 = 	Convolution2D(p1, h,nx,ny);
            GradHp11 = funGrad(Hx,yTrunc,taby,tabn,gvar,gmean,alpha,tolerance);
            GradHp1  = ConvolutionAdj2D(GradHp11,h,nx,ny);
            q1 = p1 - gamma * (Gsp21 + Gsp22+GradHp1);
            x = x - y1 + q1;
            % display 
            if(mod(k,affiche)==1)
                SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
                MAE = mean(abs(I(:)-x) );
                disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]);    
                
            end
        end
        % compute snr
        SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
        MAE = mean(abs(I(:)-x) );
end

disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function Tx = Convolution2D(x,h,nx,ny)
% 
% if(size(h,1)*size(h,2)>1)     
%     X = reshape(x,nx,ny);
%     Tx = conv2(X,h,'same');
%     Tx = Tx(:);    
% else
%     Tx = h.*x(:);
% end
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function Ttz = ConvolutionAdj2D(z,h,nx,ny)
% 
% if(size(h,1)*size(h,2)>1)
%     Z = reshape(z,ny,nx);
%     Ttz = conv2(Z,flipud(fliplr(h)),'same');
%     Ttz = Ttz(:);
% else
%     Ttz = z(:);
% end 
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function gTx = grad_gaussiandep(Tx,y,taby,tabn,gvar,gmean, alpha)  
% gTx = (alpha*Tx-y)./(gvar); 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function z = prox_poisson(coef, x,y)
% z = (x- coef + sqrt((x- coef).^2 + 4*coef.*y))./2; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function gTx = grad_gaussiangast(Tx,y,taby,tabn, gvar,gmean, alpha)   
% Tneg = (Tx < 0 ) ;
% Tpos = (Tx >= 0 ) ;
% 
% % Build quadratic approximation in Tx < 0
% cteGAST = 3/8  + gvar;
% A = cteGAST^(-3/2) * y./2 ;
% B = (2-y./sqrt(cteGAST));
% gTx(Tneg) =  A(Tneg) .* Tx(Tneg) + B(Tneg) ;
% 
% % Build exact gradient if Tx >= 0
% gTx(Tpos) =  2-y(Tpos)./sqrt(Tx(Tpos) + cteGAST) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function gTx = grad_weightedl2(Tx,y,taby,tabn, gvar,gmean, alpha)   
% a = 1; b = gvar;
% Tneg = (Tx < 0 ) ;
% Tpos = (Tx >= 0 ) ;
% 
% % Build quadratic approximation in Tx < 0
% A = 1/2.*(a.*y + b).^2 ./b^3;
% B = -1/2.*(y.*(a.*y + 2.*b)./(b^2));
% gTx(Tneg) = 2*A(Tneg).*alpha.*Tx(Tneg)+B(Tneg);
% 
% % Build exact gradient if Tx >= 0
% gTx(Tpos) = ((alpha.*Tx(Tpos)-y(Tpos)).*(a.*(alpha.*Tx(Tpos)+y(Tpos))+2*b))./(2*(a.*alpha.*Tx(Tpos)+b).^2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function z = prox_tv(coef,x,lambda,nx,ny) (x,degree, coeff,nx,ny)
% 
% u = x(1:nx*ny);
% v = x(nx*ny+1:end);
% zu = zeros(nx*ny,1);    
% zv = zu;
% coeff = coef*lambda;
% 
% sqrtuv = sqrt(u.^2 + v.^2);
% zu(sqrtuv>coeff) = (1-coeff./sqrtuv(sqrtuv>coeff)).*u(sqrtuv>coeff);    
% zv(sqrtuv>coeff) = (1-coeff./sqrtuv(sqrtuv>coeff)).*v(sqrtuv>coeff);
% z = [zu;zv];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
