function [x] = demo_article(resI, resD, resP, resR)
%   Demonstration of all experiments presented in:
% 	
% 	[2] Emilie Chouzenoux, Anna Jezierska, Jean-Christophe Pesquet, and Hugues Talbot
% 	A Convex Approach for Image Restoration with Exact Poisson-Gaussian Likelihood,
%   submitted to SIAM Journal on Imaging Science, 2015
% 	
%
%   Copyright 2013 Anna Jezierska
% 	This function computes the Poisson-Gaussian data fidelity term
% 	as described in
% 
% 
% 	[1] Anna Jezierska, Emilie Chouzenoux, Jean-Christophe Pesquet, and Hugues Talbot
% 	A primal-dual proximal splitting approach for restoring data corrupted with Poisson-Gaussian noise,
% 	International Conference on Acoustics, Speech, and Signal Processing (ICASSP),
% 	Kyoto, 25-30 March, 2012.
% 
% 	[2] Emilie Chouzenoux, Anna Jezierska, Jean-Christophe Pesquet, and Hugues Talbot
% 	A Convex Approach for Image Restoration with Exact Poisson-Gaussian Likelihood,
%   submitted to SIAM Journal on Imaging Science, 2015
% 
% 	[3] Anna Jezierska (2013).
% 	Image Restoration in the presence of Poisson-Gaussian noise.
% 	Ph.D. Thesis. Université Paris-Est
% 
% 	This software was developed by Anna Jezierska version of 03.2014
% 	(Anna.Jezierska@ibspan.waw.pl).
% 
% 	If you use this software for research purposes, you should cite
% 	the aforementioned paper in any resulting publication.
% .
% Usage example:  demo_article(2,1,3,1);

addpath Tools
addpath algo
addpath NLTV_functions
addpath PG_fid_functions
addpath Benvenuto_fid_functions
addpath LambertW
addpath Images

data_fidelity_str = {'Poiss' 'Gauss' 'GAST' 'EXP' 'SPoiss' 'WL2' 'PoissGauss' };
prior_str = {'TV' 'NLTV' 'HTV'};
image_str = {'ISBI_128_128' 'SIP' 'crym60'};
output_strII={'ISBI_range01_var09_blurG25_noisy' 'ISBI_range02_var12_blurG25_noisy' 'ISBI_range08_var60_blurG25_noisy'};
output_str={'ISBI_range02_var12_blurG25_noisy' 'SIP_var9_blurGauss25_16' 'crym60_blur_gauss_9_var05_noise_var_36'};
blur_str={'Gaussian 9x9 var = 0.5' 'Gaussian 25x25 var = 1.6'};
%max_str = [15 30 120];
max_str = [1000 1000 1000];
range_str = [0.1 0.2 0.8];
var_str = [12 9 36];
var_strII = [9 12 60];
lambda_tableII = [0.146 0.140 0.069 0.081 0.077 0.079 0.080; 
                  0.105 0.121 0.060 0.055 0.059 0.054 0.052; 
                  0.051 0.044231384 0.019 0.020 0.020 0.021 0.020];
              
lambda_tableIII = [0.164 0.197 0.093 0.096 0.095 0.099 0.093; 
                  0.105 0.121 0.060 0.055 0.059 0.054 0.052; 
                  0.043 0.075 0.02659 0.031 0.030 0.037 0.031;
                  0.148 0.136 0.07027 0.072 0.070 0.059 0.070];   
              
lambda_tableIV  = [0.338 0.218 0.155 0.156 0.153 0.133 0.154; 
                  0.243 0.164 0.115 0.114 0.117 0.101 0.11334914; 
                  0.016 0.004 0.016 0.012 0.013 0.05099 0.008;
                  0.609 0.472 0.293 0.297 0.292 0.11897 0.296];    
              
lambda_tableV  = [0.299 0.11646141 0.078 0.078 0.078 0.074921126 0.077; 
                  0.301 0.118 0.078 0.077 0.077 0.074 0.076;
                  0.050 0.037 0.052 0.030 0.03185 0.046 0.0349138007;
                  0.127 0.054 0.009 0.034 0.021859 0.020 0.0198190281];               
xmin = 0;

% ISBI
NbIt_tableII = [2.5 3 5 15 5 7 70; 
                10 9 12.5 70 12.5 12.5 70; 
                50 70 70 100 155 85 135];

% ISBI
NbIt_tableIII = [10 9 12.5 50 12.5 25 50; 
                 10 9 12.5 70 12.5 12.5 70; 
                 30 5 10 15 40 8 135];
 
% SIP            
NbIt_tableIV = [2.5 3.5 3.5 8 3.5 3.5 20; %TV
                5 5 5 5 2.5 2.5 2.5;   %NLTV
                4.5 2.0 2.0 3 9 2.0 5.5];  % HTV   10 3.5 4 5 15 3 10] (even more stable result)]  

% Crym60            
NbIt_tableV = [2 3 3.5 5 5 5 5; 
               2.5 3.5 5 5 5 4 6;
               30 5 8 8 55 8 15]; % 
           
            
%%%%%%%%%%IMAGE%%%%%%%%%%%%%%%%%%%%%%%%%%%
prompt = ['Image choice:\n' '1 - x1 \n' '2 - x2 \n' '3 - x3 \n'];
%resI = input(prompt);
image=image_str(resI);
image=num2str(cell2mat(image));
%%%%%%%%%%DATA FIDELITY%%%%%%%%%%%%%%%%%%%%
prompt = ['Data fidelity choice:\n' '1- Poiss\n' '2-Gauss\n' '3-GAST\n' '4-EXP\n' '5-SPoiss\n' '6-WL2\n' '7-PoissGauss\n' ];
%fprintf(prompt);     
%resD = input(prompt);
data_fidelity = data_fidelity_str(resD);
data_fidelity=num2str(cell2mat(data_fidelity));
%%%%%%%%%%BLUR%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch image
    case 'SIP'
        h = fspecial('gaussian', 25, 1.6);
        blur=blur_str(2);
        xmax = 12;
        range = 1;
        degraded = output_str(resI);
        varG = var_str(resI);
    case 'ISBI_128_128'
        h = fspecial('gaussian', 25, 1.6);
        blur=blur_str(2);
        prompt = ['x+ choice (range):\n' '1 - 15 \n' '2 - 30 \n' '3 - 120 \n'];
        %resR = input(prompt);
        xmax = max_str(resR);
        range = range_str(resR);
        prior = 'NLTV';
        lambda = lambda_tableII(resR,resD);
        degraded = output_strII(resR);
        varG = var_strII(resR);
    case 'crym60'
        h = fspecial('gaussian', 9, 0.5);
        blur=blur_str(1);
        xmax = 60;
        range = 1;
        degraded = output_str(resI);
        varG = var_str(resI);
end

degraded=num2str(cell2mat(degraded));
blur=num2str(cell2mat(blur));
I = range*importdata(strcat('Images/',image,'.txt'));
y = importdata(strcat('Images/',degraded,'.txt'));
y=y(:);
%%%%%%%%%%PRIOR%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (xmax ~=15 & xmax ~= 120)
    prompt = ['Regularization term choice:\n' '1 - TV \n' '2 - NLTV\n' '3 - HTV\n'];
    %resP = input(prompt);
    prior=prior_str(resP);
    prior=num2str(cell2mat(prior));
    %%%%%%%%%%LAMBDA%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (strcmp(prior,'HTV')~=1)
        switch image
            case 'SIP'
                lambda = lambda_tableIV(resP,resD);
                
            case 'ISBI_128_128'
                lambda = lambda_tableIII(resP,resD);
		
            case 'crym60'
                lambda = lambda_tableV(resP,resD);
               
        end
    else
        switch image
            case 'SIP'
                lambda(1) = lambda_tableIV(resP,resD);
                lambda(2) = lambda_tableIV(resP+1,resD);
            case 'ISBI_128_128'
                lambda(1) = lambda_tableIII(resP,resD);
                lambda(2) = lambda_tableIII(resP+1,resD);
            case 'crym60'
                lambda(1) = lambda_tableV(resP,resD);
                lambda(2) = lambda_tableV(resP+1,resD);
        end
    end
end
%%%%%%%%%%SUMMARY%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['Image: ' image ]); 
disp(['Data fidelity: ' data_fidelity ]); 
disp(['Prior term: ' prior]); 
disp(['Blur: ' blur]); 
disp(['Variance of Gaussian noise: ' num2str(varG)]); 
disp(['x+: ', num2str(xmax)]);
disp(['Lambda: ', num2str(lambda)]);
%%%%%%%%%%NOISE%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch image
            case 'SIP'
                
                NbIt = NbIt_tableIV(resP,resD);
            case 'ISBI_128_128'
                
		NbIt = NbIt_tableII(resP,resD);
            case 'crym60'
                
                NbIt = NbIt_tableV(resP,resD);
        end


alpha = 1;
m = 0;
tolerance = 3;
NbIt = NbIt*1000;
x = 0;
yTrunc = dataTruncation(data_fidelity, y, varG, m);
[nx,ny] = size(I);
x0 = ones(nx,ny);
x0 = x0(:);
x = 0;
tic
switch prior
    case 'TV'
        [V1,V2] = difference_matrices_iso(nx,ny); % TV matrices
        L = [V1 ;V2];
        gamma = getGamma(data_fidelity, L, h, yTrunc, varG, m,nx,ny);
        [MAE, SNR, x] = primaldualrecPGNLTV(x0,y,yTrunc,h,NbIt,nx,ny,varG,m,alpha,lambda,xmin,xmax,I,data_fidelity,prior,L,0,gamma,tolerance);
    case 'NLTV'
        SNRinit = 10*log10(sum(I(:).^2)/sum((I(:)-y(:)).^2));
        [L, degree, x0] = NLTVoperator(y,nx,ny,h,SNRinit,xmax);
        gamma = getGamma(data_fidelity, L, h, yTrunc, varG, m,nx,ny);
        [MAE, SNR, x] = primaldualrecPGNLTV(x0,y,yTrunc,h,NbIt,nx,ny,varG,m,alpha,lambda,xmin,xmax,I,data_fidelity,prior,L,degree,gamma,tolerance);
    case 'HTV'
        [V11,V22,V12] = difference_matrices_unser(nx,ny); % Hessian matrices
        [V1,V2] = difference_matrices_iso(nx,ny); % TV matrices
        VHes =  [V11 ;sqrt(2)*V12 ;V22];
        VTV = [V1 ;V2];
        gamma = getGammaHyb(data_fidelity, VHes,VTV, h, yTrunc, varG, m,nx,ny);
        x = primaldualrecPG_Hyb(x0,y,yTrunc,h,NbIt,nx,ny,varG,m,alpha,lambda(2),lambda(1), xmin,xmax,I,data_fidelity,prior, VHes, VTV, 0,gamma,tolerance);
end
toc
t=toc
end

